@@ -1,11 +1,9 @@
 // KMail Account
-#include <config.h>
 
 #include "kmaccount.h"
 
 #include "accountmanager.h"
 using KMail::AccountManager;
-#include "globalsettings.h"
 #include "kmacctfolder.h"
 #include "kmfoldermgr.h"
 #include "kmfiltermgr.h"
@@ -19,40 +17,43 @@ using KPIM::BroadcastStatus;
 using KPIM::ProgressItem;
 using KPIM::ProgressManager;
 
-#include <libkpimidentities/identitymanager.h>
-#include <libkpimidentities/identity.h>
+#include <kpimidentities/identitymanager.h>
+#include <kpimidentities/identity.h>
 
 using KMail::FolderJob;
 
-#include <kapplication.h>
 #include <klocale.h>
 #include <kmessagebox.h>
 #include <kdebug.h>
 #include <kconfig.h>
+#include <kconfiggroup.h>
+#include <kstringhandler.h>
+#include <knotification.h>
 
-#include <qeventloop.h>
+#include <QList>
+#include <QEventLoop>
+#include <QByteArray>
 
-#include <stdlib.h>
+#include <cstdlib>
 #include <unistd.h>
-#include <errno.h>
+#include <cerrno>
 
-#include <assert.h>
+#include <cassert>
 
 //----------------------
 #include "kmaccount.moc"
 
 //-----------------------------------------------------------------------------
 KMPrecommand::KMPrecommand(const QString &precommand, QObject *parent)
-  : QObject(parent), mPrecommand(precommand)
+  : QObject( parent ), mPrecommand( precommand )
 {
   BroadcastStatus::instance()->setStatusMsg(
-      i18n("Executing precommand %1").arg(precommand ));
+      i18n("Executing precommand %1", precommand ));
 
-  mPrecommandProcess.setUseShell(true);
-  mPrecommandProcess << precommand;
+  mPrecommandProcess.setShellCommand(precommand);
 
-  connect(&mPrecommandProcess, SIGNAL(processExited(KProcess *)),
-          SLOT(precommandExited(KProcess *)));
+  connect(&mPrecommandProcess, SIGNAL(finished(int, QProcess::ExitStatus)),
+          SLOT(precommandExited(int, QProcess::ExitStatus)));
 }
 
 //-----------------------------------------------------------------------------
@@ -64,21 +65,27 @@ KMPrecommand::~KMPrecommand()
 //-----------------------------------------------------------------------------
 bool KMPrecommand::start()
 {
-  bool ok = mPrecommandProcess.start( KProcess::NotifyOnExit );
-  if (!ok) KMessageBox::error(0, i18n("Could not execute precommand '%1'.")
-    .arg(mPrecommand));
+  mPrecommandProcess.start();
+  const bool ok = mPrecommandProcess.waitForStarted();
+  if ( !ok ) {
+    KNotification::event( "mail-check-error",
+                          i18n( "Could not execute precommand '%1'.", mPrecommand ) );
+  }
+
   return ok;
 }
 
 
 //-----------------------------------------------------------------------------
-void KMPrecommand::precommandExited(KProcess *p)
+void KMPrecommand::precommandExited(int exitCode, QProcess::ExitStatus)
 {
-  int exitCode = p->normalExit() ? p->exitStatus() : -1;
-  if (exitCode)
-    KMessageBox::error(0, i18n("The precommand exited with code %1:\n%2")
-      .arg(exitCode).arg(strerror(exitCode)));
-  emit finished(!exitCode);
+  if (exitCode != 0) {
+    KNotification::event( "mail-check-error",
+                          i18n( "The precommand exited with code %1:\n%2",
+                                 exitCode, strerror( exitCode ) ) );
+  }
+
+  emit finished(exitCode == 0);
 }
 
 
@@ -93,11 +100,16 @@ KMAccount::KMAccount(AccountManager* aOwner, const QString& aName, uint id)
     mExclude(false),
     mCheckingMail(false),
     mPrecommandSuccess(true),
+    mUseDefaultIdentity(true),
     mHasInbox(false),
     mMailCheckProgressItem(0),
-    mIdentityId(0)
+    mPrecommandEventLoop( 0 )
 {
   assert(aOwner != 0);
+  mIdentityId = kmkernel->identityManager()->defaultIdentity().uoid();
+
+  connect( kmkernel->identityManager(), SIGNAL( changed() ),
+           this, SLOT( slotIdentitiesChanged() ) );
 }
 
 void KMAccount::init() {
@@ -132,7 +144,7 @@ void KMAccount::clearPasswd()
 void KMAccount::setFolder(KMFolder* aFolder, bool addAccount)
 {
   if(!aFolder) {
-    //kdDebug(5006) << "KMAccount::setFolder() : aFolder == 0" << endl;
+    //kDebug() << "aFolder == 0";
     mFolder = 0;
     return;
   }
@@ -142,19 +154,23 @@ void KMAccount::setFolder(KMFolder* aFolder, bool addAccount)
 
 
 //-----------------------------------------------------------------------------
-void KMAccount::readConfig(KConfig& config)
+void KMAccount::readConfig(KConfigGroup& config)
 {
   QString folderName;
   mFolder = 0;
   folderName = config.readEntry("Folder");
-  setCheckInterval(config.readNumEntry("check-interval", 0));
+  setCheckInterval(config.readEntry("check-interval", 0 ) );
   setTrash(config.readEntry("trash", kmkernel->trashFolder()->idString()));
-  setCheckExclude(config.readBoolEntry("check-exclude", false));
-  setPrecommand(config.readPathEntry("precommand"));
-  setIdentityId(config.readNumEntry("identity-id", 0));
-  if (!folderName.isEmpty())
-  {
-    setFolder(kmkernel->folderMgr()->findIdString(folderName), true);
+  setCheckExclude(config.readEntry("check-exclude", false ) );
+  setPrecommand(config.readPathEntry("precommand", QString()));
+
+  uint defaultIdentity = kmkernel->identityManager()->defaultIdentity().uoid();
+  setIdentityId( config.readEntry( "identity-id", defaultIdentity ) );
+  mUseDefaultIdentity = config.readEntry( "use-default-identity", true );
+  slotIdentitiesChanged();
+
+  if ( !folderName.isEmpty() ) {
+    setFolder (kmkernel->folderMgr()->findIdString( folderName ), true );
   }
 
   if (mInterval == 0)
@@ -175,33 +191,30 @@ void KMAccount::readTimerConfig()
 }
 
 //-----------------------------------------------------------------------------
-void KMAccount::writeConfig(KConfig& config)
+void KMAccount::writeConfig(KConfigGroup& config)
 {
   // ID, Name
   KAccount::writeConfig(config);
 
-  config.writeEntry("Type", type());
-  config.writeEntry("Folder", mFolder ? mFolder->idString() : QString::null);
+  config.writeEntry("Type", KAccount::nameForType( type() ) );
+  config.writeEntry("Folder", mFolder ? mFolder->idString() : QString());
   config.writeEntry("check-interval", mInterval);
   config.writeEntry("check-exclude", mExclude);
   config.writePathEntry("precommand", mPrecommand);
   config.writeEntry("trash", mTrash);
-  if ( mIdentityId && mIdentityId != kmkernel->identityManager()->defaultIdentity().uoid() )
-    config.writeEntry("identity-id", mIdentityId);
-  else
-    config.deleteEntry("identity-id");
+  config.writeEntry( "use-default-identity", mUseDefaultIdentity );
+  config.writeEntry( "identity-id", mIdentityId );
 }
 
 
 //-----------------------------------------------------------------------------
 void KMAccount::sendReceipt(KMMessage* aMsg)
 {
-  KConfig* cfg = KMKernel::config();
   bool sendReceipts;
 
-  KConfigGroupSaver saver(cfg, "General");
+  KConfigGroup cfg( KMKernel::config(), "General" );
 
-  sendReceipts = cfg->readBoolEntry("send-receipts", false);
+  sendReceipts = cfg.readEntry("send-receipts", false );
   if (!sendReceipts) return;
 
   KMMessage *newMsg = aMsg->createDeliveryReceipt();
@@ -221,7 +234,7 @@ bool KMAccount::processNewMsg(KMMessage* aMsg)
 
   // Save this one for readding
   KMFolderCachedImap* parent = 0;
-  if( type() == "cachedimap" )
+  if( type() == KAccount::DImap )
     parent = static_cast<KMFolderCachedImap*>( aMsg->storage() );
 
   // checks whether we should send delivery receipts
@@ -231,18 +244,18 @@ bool KMAccount::processNewMsg(KMMessage* aMsg)
   // Set status of new messages that are marked as old to read, otherwise
   // the user won't see which messages newly arrived.
   // This is only valid for pop accounts and produces wrong stati for imap.
-  if ( type() != "cachedimap" && type() != "imap" ) {
-    if ( aMsg->isOld() )
-      aMsg->setStatus(KMMsgStatusUnread);  // -sanders
-    //    aMsg->setStatus(KMMsgStatusRead);
+  if ( type() != KAccount::DImap && type() != KAccount::Imap ) {
+    if ( aMsg->status().isOld() )
+      aMsg->setStatus( MessageStatus::statusUnread() );  // -sanders
+    //    aMsg->setStatus( MessageStatus::statusRead() );
     else
-      aMsg->setStatus(KMMsgStatusNew);
+      aMsg->setStatus( MessageStatus::statusNew() );
   }
 /*
 QFile fileD0( "testdat_xx-kmaccount-0" );
-if( fileD0.open( IO_WriteOnly ) ) {
+if( fileD0.open( QIODevice::WriteOnly ) ) {
     QDataStream ds( &fileD0 );
-    ds.writeRawBytes( aMsg->asString(), aMsg->asString().length() );
+    ds.writeRawData( aMsg->asString(), aMsg->asString().length() );
     fileD0.close();  // If data is 0 we just create a zero length file.
 }
 */
@@ -250,14 +263,14 @@ if( fileD0.open( IO_WriteOnly ) ) {
 
   processResult = kmkernel->filterMgr()->process(aMsg,KMFilterMgr::Inbound,true,id());
   if (processResult == 2) {
-    perror("Critical error: Unable to collect mail (out of space?)");
+    kError() << "Critical error: Unable to collect mail (out of space?)";
     KMessageBox::information(0,(i18n("Critical error: "
       "Unable to collect mail: ")) + QString::fromLocal8Bit(strerror(errno)));
     return false;
   }
   else if (processResult == 1)
   {
-    if( type() == "cachedimap" )
+    if( type() == KAccount::DImap )
       ; // already done by caller: parent->addMsgInternal( aMsg, false );
     else {
       // TODO: Perhaps it would be best, if this if was handled by a virtual
@@ -266,14 +279,14 @@ if( fileD0.open( IO_WriteOnly ) ) {
       rc = mFolder->addMsg(aMsg);
 /*
 QFile fileD0( "testdat_xx-kmaccount-1" );
-if( fileD0.open( IO_WriteOnly ) ) {
+if( fileD0.open( QIODevice::WriteOnly ) ) {
     QDataStream ds( &fileD0 );
-    ds.writeRawBytes( aMsg->asString(), aMsg->asString().length() );
+    ds.writeRawData( aMsg->asString(), aMsg->asString().length() );
     fileD0.close();  // If data is 0 we just create a zero length file.
 }
 */
       if (rc) {
-        perror("failed to add message");
+        kError() << "Failed to add message!";
         KMessageBox::information(0, i18n("Failed to add message:\n") +
                                  QString(strerror(rc)));
         return false;
@@ -287,8 +300,8 @@ if( fileD0.open( IO_WriteOnly ) ) {
   // Count number of new messages for each folder
   QString folderId;
   if ( processResult == 1 ) {
-    folderId = ( type() == "cachedimap" ) ? parent->folder()->idString()
-                                          : mFolder->idString();
+    folderId = ( type() == KAccount::DImap ) ? parent->folder()->idString()
+                                             : mFolder->idString();
   }
   else {
     folderId = aMsg->parent()->idString();
@@ -312,28 +325,30 @@ int KMAccount::checkInterval() const
 {
   if ( mInterval <= 0 )
     return mInterval;
-  return QMAX( mInterval, GlobalSettings::self()->minimumCheckInterval() );
+  return qMax( mInterval, GlobalSettings::self()->minimumCheckInterval() );
 }
 
 //----------------------------------------------------------------------------
 void KMAccount::deleteFolderJobs()
 {
-  mJobList.setAutoDelete(true);
+  qDeleteAll( mJobList );
   mJobList.clear();
-  mJobList.setAutoDelete(false);
 }
 
 //----------------------------------------------------------------------------
 void KMAccount::ignoreJobsForMessage( KMMessage* msg )
 {
   //FIXME: remove, make folders handle those
-  for( QPtrListIterator<FolderJob> it(mJobList); it.current(); ++it ) {
-    if ( it.current()->msgList().first() == msg) {
-      FolderJob *job = it.current();
-      mJobList.remove( job );
+  QList<FolderJob*>::iterator it;
+  for( it = mJobList.begin(); it != mJobList.end(); ) {
+    if ( (*it)->msgList().first() == msg) {
+      FolderJob *job = (*it);
+      it = mJobList.erase( it );
       delete job;
       break;
     }
+    else
+      ++it;
   }
 }
 
@@ -350,7 +365,7 @@ void KMAccount::installTimer()
   if (mInterval <= 0) return;
   if(!mTimer)
   {
-    mTimer = new QTimer(0, "mTimer");
+    mTimer = new QTimer();
     connect(mTimer,SIGNAL(timeout()),SLOT(mailCheck()));
   }
   else
@@ -375,18 +390,30 @@ bool KMAccount::runPrecommand(const QString &precommand)
   if ( precommand.isEmpty() )
     return true;
 
+  // Don't do anything if there is a running pre-command
+  if ( mPrecommandEventLoop != 0 )
+    return true;
+
   KMPrecommand precommandProcess(precommand, this);
 
   BroadcastStatus::instance()->setStatusMsg(
-      i18n("Executing precommand %1").arg(precommand ));
+      i18n("Executing precommand %1", precommand ));
 
   connect(&precommandProcess, SIGNAL(finished(bool)),
           SLOT(precommandExited(bool)));
 
-  kdDebug(5006) << "Running precommand " << precommand << endl;
-  if (!precommandProcess.start()) return false;
+  kDebug() << "Running precommand" << precommand;
+  if ( !precommandProcess.start() )
+    return false;
 
-  kapp->eventLoop()->enterLoop();
+  // Start an event loop. This makes sure GUI events are still processed while
+  // the precommand is running (which may take a while).
+  // The exec call will block until the event loop is exited, which happens in
+  // precommandExited().
+  mPrecommandEventLoop = new QEventLoop();
+  mPrecommandEventLoop->exec();
+  delete mPrecommandEventLoop;
+  mPrecommandEventLoop = 0;
 
   return mPrecommandSuccess;
 }
@@ -395,7 +422,17 @@ bool KMAccount::runPrecommand(const QString &precommand)
 void KMAccount::precommandExited(bool success)
 {
   mPrecommandSuccess = success;
-  kapp->eventLoop()->exitLoop();
+  if ( mPrecommandEventLoop )  // don't crash when kmail exits while mPrecommandEventLoop runs
+    mPrecommandEventLoop->exit();
+}
+
+//-----------------------------------------------------------------------------
+void KMAccount::slotIdentitiesChanged()
+{
+  // Fall back to the default identity if the one used currently is invalid
+  if ( kmkernel->identityManager()->identityForUoid( mIdentityId ).isNull() ) {
+    mIdentityId = kmkernel->identityManager()->defaultIdentity().uoid();
+  }
 }
 
 //-----------------------------------------------------------------------------
@@ -406,49 +443,38 @@ void KMAccount::mailCheck()
 
   if ( kmkernel ) {
     AccountManager *acctmgr = kmkernel->acctMgr();
-    if ( acctmgr )
-      acctmgr->singleCheckMail(this, false);
+    if ( acctmgr ) {
+      acctmgr->singleCheckMail( this, false );
+    }
   }
 }
 
 //-----------------------------------------------------------------------------
 void KMAccount::sendReceipts()
 {
-  QValueList<KMMessage*>::Iterator it;
+  QList<KMMessage*>::Iterator it;
   for(it = mReceipts.begin(); it != mReceipts.end(); ++it)
     kmkernel->msgSender()->send(*it); //might process events
   mReceipts.clear();
 }
 
-//-----------------------------------------------------------------------------
-QString KMAccount::encryptStr(const QString &aStr)
-{
-  QString result;
-  for (uint i = 0; i < aStr.length(); i++)
-    /* yes, no typo. can't encode ' ' or '!' because
-       they're the unicode BOM. stupid scrambling. stupid. */
-    result += (aStr[i].unicode() <= 0x21 ) ? aStr[i] :
-      QChar(0x1001F - aStr[i].unicode());
-  return result;
-}
-
 //-----------------------------------------------------------------------------
 QString KMAccount::importPassword(const QString &aStr)
 {
   unsigned int i, val;
   unsigned int len = aStr.length();
-  QCString result;
-  result.resize(len+1);
+  QByteArray result;
+  result.resize(len);
 
   for (i=0; i<len; i++)
   {
-    val = aStr[i] - ' ';
+    val = aStr[i].toLatin1() - ' ';
     val = (255-' ') - val;
     result[i] = (char)(val + ' ');
   }
   result[i] = '\0';
 
-  return encryptStr(result);
+  return KStringHandler::obscure(result);
 }
 
 void KMAccount::invalidateIMAPFolders()
@@ -467,6 +493,7 @@ void KMAccount::pseudoAssign( const KMAccount * a ) {
   setPrecommand( a->precommand() );
   setTrash( a->trash() );
   setIdentityId( a->identityId() );
+  setUseDefaultIdentity( a->useDefaultIdentity() );
 }
 
 //-----------------------------------------------------------------------------
@@ -478,11 +505,8 @@ void KMAccount::checkDone( bool newmail, CheckStatus status )
   if (mTimer)
     mTimer->start( checkInterval() * 60000 );
   if ( mMailCheckProgressItem ) {
-    // set mMailCheckProgressItem = 0 before calling setComplete() to prevent
-    // a race condition
-    ProgressItem *savedMailCheckProgressItem = mMailCheckProgressItem;
+    mMailCheckProgressItem->setComplete(); // that will delete it
     mMailCheckProgressItem = 0;
-    savedMailCheckProgressItem->setComplete(); // that will delete it
   }
 
   emit newMailsProcessed( mNewInFolder );
@@ -490,8 +514,16 @@ void KMAccount::checkDone( bool newmail, CheckStatus status )
   mNewInFolder.clear();
 }
 
+uint KMAccount::identityId() const
+{
+  if ( mUseDefaultIdentity )
+    return kmkernel->identityManager()->defaultIdentity().uoid();
+  else
+    return mIdentityId;
+}
+
 //-----------------------------------------------------------------------------
-void KMAccount::addToNewInFolder( QString folderId, int num )
+void KMAccount::addToNewInFolder( const QString &folderId, int num )
 {
   if ( mNewInFolder.find( folderId ) == mNewInFolder.end() )
     mNewInFolder[folderId] = num;
