@@ -34,14 +34,13 @@
 
 #include "imapaccountbase.h"
 
-#include <qguardedptr.h>
+#include <QList>
+#include <QPointer>
 
 class KMFolderCachedImap;
-class KMFolderTreeItem;
 class KMFolder;
 namespace KMail {
-  class FolderJob;
-  class ImapJob;
+    class ImapJob;
   class CachedImapJob;
 }
 using KMail::ImapJob;
@@ -72,14 +71,8 @@ public:
   /**
    * Inherited methods.
    */
-  virtual QString type() const;
   virtual void processNewMail( bool interactive );
 
-  /**
-   * Kill all jobs related the the specified folder
-   */
-  void killJobsForItem(KMFolderTreeItem * fti);
-
   /**
    * Kill the slave if any jobs are active
    */
@@ -96,8 +89,8 @@ public:
   virtual void setImapFolder(KMFolderCachedImap *);
   KMFolderCachedImap* imapFolder() const { return mFolder; }
 
-  virtual void readConfig( /*const*/ KConfig/*Base*/ & config );
-  virtual void writeConfig( KConfig/*Base*/ & config ) /*const*/;
+  virtual void readConfig( KConfigGroup & config );
+  virtual void writeConfig( KConfigGroup & config );
 
   /**
    * Invalidate the local cache.
@@ -106,23 +99,23 @@ public:
   virtual void invalidateIMAPFolders( KMFolderCachedImap* );
 
   /**
-   * Remember that a folder got explicitely deleted - including all child folders
+   * Remember that a folder got explicitly deleted - including all child folders
    */
   void addDeletedFolder( KMFolder* folder );
 
   /**
-   * Remember that a folder got explicitely deleted - NOT including all child folders
+   * Remember that a folder got explicitly deleted - NOT including all child folders
    * This is used when renaming a folder.
    */
   void addDeletedFolder( const QString& imapPath );
 
   /**
-   * Ask if a folder was explicitely deleted in this session
+   * Ask if a folder was explicitly deleted in this session
    */
   bool isDeletedFolder( const QString& subFolderPath ) const;
 
   /**
-   * Ask if a folder was explicitely deleted in a previous session
+   * Ask if a folder was explicitly deleted in a previous session
    */
   bool isPreviouslyDeletedFolder( const QString& subFolderPath ) const;
 
@@ -179,11 +172,11 @@ public:
   /**
    * Returns the root folder of this account
    */
-  virtual FolderStorage* const rootFolder() const;
-  
+  virtual FolderStorage* rootFolder() const;
+
   /** return if the account passed the annotation test  */
-  bool annotationCheckPassed(){ return mAnnotationCheckPassed;};
-  void setAnnotationCheckPassed( bool a ){ mAnnotationCheckPassed = a; };
+  bool annotationCheckPassed(){ return mAnnotationCheckPassed;}
+  void setAnnotationCheckPassed( bool a ){ mAnnotationCheckPassed = a; }
 
   /** Describes whether the account is a groupware account. */
   enum GroupwareType
@@ -192,6 +185,7 @@ public:
     GroupwareKolab,    ///< A Kolab groupware account
     GroupwareScalix    ///< A Scalix groupware account
   };
+  Q_ENUMS( GroupwareType )
 
   void setGroupwareType( GroupwareType type ){ mGroupwareType = type; }
   GroupwareType groupwareType() const { return mGroupwareType; }
@@ -212,11 +206,11 @@ protected slots:
   virtual void slotCheckQueuedFolders();
 
 private:
-  QValueList<KMFolderCachedImap*> killAllJobsInternal( bool disconnectSlave );
+  QList<KMFolderCachedImap*> killAllJobsInternal( bool disconnectSlave );
   void processNewMail( KMFolderCachedImap* folder, bool recurse );
 
 private:
-  QPtrList<CachedImapJob> mJobList;
+  QList<CachedImapJob*> mJobList;
   KMFolderCachedImap *mFolder;
   QStringList mDeletedFolders; // folders deleted in this session
   QStringList mPreviouslyDeletedFolders; // folders deleted in a previous session
