@@ -21,7 +21,6 @@
 #define kmfolder_h
 
 // for large file support
-#include <config.h>
 
 #include "kmfoldernode.h"
 #include "kmfoldertype.h"
@@ -32,20 +31,25 @@
 using KMail::FolderJob;
 #include "mailinglist-magic.h"
 using KMail::MailingList;
-#include "kmaccount.h" // for AccountList
+//#include "kmaccount.h" // for AccountList
+class KMAccount;
+typedef QList<KMAccount*> AccountList;
 
 #include "mimelib/string.h"
+#include <kshortcut.h>
+
+#include <QList>
+
 
-#include <qptrvector.h>
 #include <sys/types.h>
 #include <stdio.h>
-#include <kshortcut.h>
+
+#define KMFOLDER_CONFIG_GROUP_NAME_PREFIX "Folder-"
+#define KMFOLDER_CONFIG_GROUP_NAME_PREFIX_LEN 7
 
 class KMMessage;
 class KMFolderDir;
-class QTimer;
 class FolderStorage;
-class KMFolderTreeItem;
 class KMFolderJob;
 
 namespace KMail {
@@ -53,7 +57,11 @@ namespace KMail {
 }
 using KMail::AttachmentStrategy;
 
-typedef QValueList<Q_UINT32> SerNumList;
+namespace QIndicate {
+  class Indicator;
+}
+
+typedef QList<quint32> SerNumList;
 
 /** Mail folder.
  * (description will be here).
@@ -79,7 +87,7 @@ public:
    * the folder is free-standing (/var/spool/mail/foo), this is used for the full path to
    * the folder's storage location.
    * @param aFolderType The type of folder to create.
-   * @param withIndex Wether this folder has an index. No-index folders are
+   * @param withIndex Whether this folder has an index. No-index folders are
    * those used by KMail internally, the Outbox, and those of local spool accounts,
    * for example.
    * @param exportedSernums whether this folder exports its serial numbers to
@@ -114,6 +122,7 @@ public:
   bool isDrafts() {
     return KMKernel::self()->folderIsDrafts( this );
   }
+
   /** Returns true if this folder is the templates folder of the local account,
     or is configured to be the templates folder of any of the users identities */
   bool isTemplates() {
@@ -123,14 +132,14 @@ public:
   void setAcctList( AccountList* list ) { mAcctList = list; }
   AccountList* acctList() { return mAcctList; }
 
-  /** Returns TRUE if accounts are associated with this folder. */
+  /** Returns true if accounts are associated with this folder. */
   bool hasAccounts() const { return (mAcctList != 0); }
 
   /** This is used by the storage to read the folder specific configuration */
-  void readConfig( KConfig* config );
+  void readConfig( KConfigGroup & config );
 
   /** This is used by the storage to save the folder specific configuration */
-  void writeConfig( KConfig* config ) const;
+  void writeConfig( KConfigGroup & config ) const;
 
   FolderStorage* storage() { return mStorage; }
   /** if the folder is const, the storage should be as well */
@@ -146,7 +155,13 @@ public:
   QString location() const;
 
   /** Returns full path to index file */
-  QString indexLocation() const;
+  virtual QString indexLocation() const;
+
+  /** Returns full path to 'ids' file */
+  virtual QString idsLocation() const;
+
+  /** Returns full path to 'sorted' file */
+  virtual QString sortedLocation() const;
 
   /** Returns full path to sub directory file */
   QString subdirLocation() const;
@@ -155,6 +170,21 @@ public:
       0 if no such directory exists */
   KMFolderDir* child() const
     { return mChild; }
+  
+  /**
+   * Returns true if the specified folder is a descendant of this folder
+   * (if fld is somewhere in this folder's children subtree).
+   * Returns false otherwise (or if fld is null).
+   * 
+   * This function assumes that fld, if not null, is a valid pointer to
+   * a folder that can be dereferenced.
+   */
+  bool hasDescendant( KMFolder *fld ) const;
+
+  /**
+   * Returns the parent folder, if any.
+   */
+  KMFolder * ownerFolder() const;
 
   /** Create a child folder directory and associates it with this folder */
   KMFolderDir* createChildFolder();
@@ -196,9 +226,9 @@ public:
    * for each derived KMFolder).
    */
   FolderJob* createJob( KMMessage *msg, FolderJob::JobType jt = FolderJob::tGetMessage,
-                        KMFolder *folder = 0, QString partSpecifier = QString::null,
+                        KMFolder *folder = 0, const QString &partSpecifier = QString(),
                         const AttachmentStrategy *as = 0 ) const;
-  FolderJob* createJob( QPtrList<KMMessage>& msgList, const QString& sets,
+  FolderJob* createJob( QList<KMMessage*>& msgList, const QString& sets,
                         FolderJob::JobType jt = FolderJob::tGetMessage,
                         KMFolder *folder = 0 ) const;
 
@@ -218,7 +248,7 @@ public:
   /** Detach message from this folder. Usable to call addMsg() afterwards.
     Loads the message if it is not loaded up to now. */
   KMMessage* take(int idx);
-  void take(QPtrList<KMMessage> msgList);
+  void takeMessages(const QList<KMMessage*>& msgList);
 
   /** Add the given message to the folder. Usually the message
     is added at the end of the folder. Returns zero on success and
@@ -237,7 +267,7 @@ public:
    * Adds the given messages to the folder. Behaviour is identical
    * to addMsg(msg)
    */
-  int addMsg(QPtrList<KMMessage>&, QValueList<int>& index_return);
+  int addMessages(QList<KMMessage*>&, QList<int>& index_return);
 
   /** Called by derived classes implementation of addMsg.
       Emits msgAdded signals */
@@ -245,7 +275,7 @@ public:
 
   /** Remove (first occurrence of) given message from the folder. */
   void removeMsg(int i, bool imapQuiet = false);
-  void removeMsg(QPtrList<KMMessage> msgList, bool imapQuiet = false);
+  void removeMessages(QList<KMMessage*> msgList, bool imapQuiet = false);
 
   /** Delete messages in the folder that are older than days. Return the
    * number of deleted messages. */
@@ -256,7 +286,7 @@ public:
     code on failure. The index of the new message is stored in index_return
     if given. */
   int moveMsg(KMMessage* msg, int* index_return = 0);
-  int moveMsg(QPtrList<KMMessage>, int* index_return = 0);
+  int moveMsg(QList<KMMessage*>, int* index_return = 0);
 
   /** Returns the index of the given message or -1 if not found. */
   int find(const KMMsgBase* msg) const;
@@ -273,25 +303,31 @@ public:
   int countUnreadRecursive();
 
   /** Called by KMMsgBase::setStatus when status of a message has changed
-      required to keep the number unread messages variable current. */
-  void msgStatusChanged( const KMMsgStatus oldStatus,
-                         const KMMsgStatus newStatus,
+      required to keep the number of unread messages variable current. */
+  void msgStatusChanged( const MessageStatus& oldStatus,
+                         const MessageStatus& newStatus,
                          int idx);
 
+  /** Called by KMMsgBase::setTagList(). Emits the msgHeaderChanged signal. */
+  void msgTagListChanged( int idx);
+
   /** Open folder for access.
-    Does nothing if the folder is already opened. To reopen a folder
-    call close() first.
+    open() and close() use reference counting.
     Returns zero on success and an error code equal to the c-library
-    fopen call otherwise (errno). */
-  int open(const char *owner);
+    fopen call otherwise (errno).
+    @p owner is for debugging. */
+  int open( const char *owner );
 
   /** Check folder for permissions
-    Returns zero if readable and writable. */
-  int canAccess();
+    @return true if readable and writable. */
+  virtual bool canAccess() const;
 
-  /** Close folder. If force is true the files are closed even if
-    others still use it (e.g. other mail reader windows). */
-  void close(const char *owner, bool force=false);
+  /** Close folder.
+    open() and close() use reference counting.
+    If @p force is true the files are closed even if
+    others are still using it (e.g. other mail reader windows).
+    @p owner is for debugging. */
+  void close( const char *owner, bool force=false );
 
   /** fsync buffers to disk */
   void sync();
@@ -353,24 +389,29 @@ public:
 
   /** Is the folder read-only? */
   bool isReadOnly() const;
+  /** Can we write into and delete from this folder (on IMAP that's not necessarily !isReadOnly()) */
+  bool isWritable() const;
+
+  /** Can messages in this folder be deleted? */
+  bool canDeleteMessages() const;
 
   /** Returns true if the folder is a kmail system folder. These are
     the folders 'inbox', 'outbox', 'sent', 'trash', 'drafts', 'templates'.
     The name of these folders is nationalized in the folder display and
     they cannot have accounts associated. Deletion is also forbidden. Etc. */
   bool isSystemFolder() const { return mIsSystemFolder; }
-  void setSystemFolder(bool itIs) { mIsSystemFolder=itIs; }
+  void setSystemFolder( bool itIs );
 
   /** Returns the label of the folder for visualization. */
   virtual QString label() const;
-  void setLabel( const QString& l ) { mLabel = l; }
+  void setLabel( const QString& l );
 
   /** Set the label that is used as a system default */
   virtual QString systemLabel() const { return mSystemLabel; }
-  void setSystemLabel( const QString& l ) { mSystemLabel = l; }
+  void setSystemLabel( const QString& l );
 
   /** URL of the node for visualization purposes. */
-  virtual QString prettyURL() const;
+  virtual QString prettyUrl() const;
 
   /** Returns true if this folder is associated with a mailing-list. */
   void setMailingListEnabled( bool enabled );
@@ -381,6 +422,11 @@ public:
   { return mMailingList; }
   QString mailingListPostAddress() const;
 
+  /** Get / set whether the default identity should be used instead of the
+   *  identity specified by setIdentity(). */
+  void setUseDefaultIdentity( bool useDefaultIdentity );
+  bool useDefaultIdentity() const { return mUseDefaultIdentity; }
+
   void setIdentity(uint identity);
   uint identity() const;
 
@@ -398,6 +444,12 @@ public:
   /** Returns a string that can be used to identify this folder */
   QString idString() const;
 
+  /**
+   * Returns the config group name for this folder
+   */
+  QString configGroupName() const
+    { return KMFOLDER_CONFIG_GROUP_NAME_PREFIX + idString(); }
+
   /**
    * Set whether this folder automatically expires messages.
    */
@@ -482,10 +534,10 @@ public:
   int writeIndex( bool createEmptyIndex = false );
 
   /** Set the status of the message at index @p idx to @p status. */
-  void setStatus(int idx, KMMsgStatus status, bool toggle=false);
+  void setStatus(int idx, const MessageStatus& status, bool toggle=false);
 
   /** Set the status of the message(s) in the QValueList @p ids to @p status. */
-  void setStatus(QValueList<int>& ids, KMMsgStatus status, bool toggle=false);
+  void setStatus(QList<int>& ids, const MessageStatus& status, bool toggle=false);
 
   /** Icon related methods */
   bool useCustomIcons() const { return mUseCustomIcons; }
@@ -511,12 +563,18 @@ public:
   bool putRepliesInSameFolder() const { return mPutRepliesInSameFolder; }
   void setPutRepliesInSameFolder( bool b ) { mPutRepliesInSameFolder = b; }
 
+  /**
+   * Returns true if this folder should be hidden from all folder selection dialogs
+   */
+  bool hideInSelectionDialog() const { return mHideInSelectionDialog; }
+  void setHideInSelectionDialog( bool hide ) { mHideInSelectionDialog = hide; }
+
   /**
    * Returns true if the user doesn't want to get notified about new mail
    * in this folder.
    */
   bool ignoreNewMail() const { return mIgnoreNewMail; }
-  void setIgnoreNewMail( bool b ) { mIgnoreNewMail = b; }
+  void setIgnoreNewMail( bool b );
 
   const KShortcut &shortcut() const { return mShortcut; }
   void setShortcut( const KShortcut& );
@@ -541,6 +599,10 @@ signals:
    * discard any messages */
   void closed();
 
+  /** Emitted at the end of a compaction job step.
+   *  in order to reload the views. */
+  void compacted();
+
   /** Emitted when the contents of a folder have been cleared
      (new search in a search folder, for example) */
   void cleared();
@@ -559,18 +621,18 @@ signals:
   void shortcutChanged( KMFolder * );
 
   /** Emitted before a message is removed from the folder. */
-  void msgRemoved(KMFolder*, Q_UINT32 sernum);
+  void msgRemoved(KMFolder*, quint32 sernum);
 
   /** Emitted after a message is removed from the folder. */
-  void msgRemoved( int idx, QString msgIdMD5 );
+  void msgRemoved( int idx, const QString& msgIdMD5 );
   void msgRemoved(KMFolder*);
 
   /** Emitted when a message is added from the folder. */
   void msgAdded(int idx);
-  void msgAdded(KMFolder*, Q_UINT32 sernum);
+  void msgAdded(KMFolder*, quint32 sernum);
 
   /** Emitted, when the status of a message is changed */
-  void msgChanged(KMFolder*, Q_UINT32 sernum, int delta);
+  void msgChanged(KMFolder*, quint32 sernum, int delta);
 
   /** Emitted when a field of the header of a specific message changed. */
   void msgHeaderChanged(KMFolder*, int);
@@ -590,6 +652,9 @@ signals:
   /** Emitted when the folder's size changes. */
   void folderSizeChanged( KMFolder * );
 
+  /** Emitted when the no content state changed. */
+  void noContentChanged();
+
 public slots:
   /** Incrementally update the index if possible else call writeIndex */
   int updateIndex();
@@ -603,11 +668,24 @@ public slots:
   void reallyAddCopyOfMsg(KMMessage* aMsg);
 
 private slots:
+
   /** The type of contents of this folder changed. Do what is needed. */
   void slotContentsTypeChanged( KMail::FolderContentsType type );
+
   /** Triggered by the storage when its size changed. */
   void slotFolderSizeChanged();
 
+  /** Triggered when the user changes the identity configuration.
+   *  It ensures that the folder identity stays valid by falling back
+   *  to the default identity if the current one became invalid. */
+  void slotIdentitiesChanged();
+
+  void updateIndicatorCount();
+  void updateIndicatorText();
+  void updateIndicatorIcon();
+
+  void slotIndicatorClicked();
+
 private:
   FolderStorage* mStorage;
   KMFolderDir* mChild;
@@ -616,7 +694,7 @@ private:
   bool mExportsSernums :1;
   bool mMoveInProgress :1;
 
-  /** nationalized label or QString::null (then name() should be used) */
+  /** nationalized label or QString() (then name() should be used) */
   QString mLabel;
   QString mSystemLabel;
 
@@ -640,6 +718,7 @@ private:
 
   AccountList* mAcctList;
 
+  bool mUseDefaultIdentity;
   uint mIdentity;
 
   /** name of the field that is used for "From" in listbox */
@@ -648,13 +727,17 @@ private:
   /** Should replies to messages in this folder be put in here? */
   bool mPutRepliesInSameFolder;
 
+  /** Should this folder be hidden in the folder selection dialog? */
+  bool mHideInSelectionDialog;
+
   /** Should new mail in this folder be ignored? */
   bool mIgnoreNewMail;
 
   /** shortcut associated with this folder or null, if none is configured. */
   KShortcut mShortcut;
-};
 
+  QIndicate::Indicator* mIndicator;
+};
 
 /**
    RAII for KMFolder::open() / close().
@@ -662,12 +745,12 @@ private:
    Usage: const KMFolderOpener opener( folder );
 */
 class KMFolderOpener {
-  KMFolder* mFolder;
-  const char* const mOwner;
+  KMFolder * mFolder;
+  const char * const mOwner;
   int mOpenRc;
 
 public:
-  KMFolderOpener( KMFolder* folder, const char* const owner )
+  KMFolderOpener( KMFolder *folder, const char *owner )
    : mFolder( folder )
    , mOwner( owner )
   {
@@ -681,14 +764,13 @@ public:
       mFolder->close( mOwner );
   }
 
-  KMFolder* folder() const { return mFolder; }
-
+  KMFolder * folder() const { return mFolder; }
+  
   int openResult() const { return mOpenRc; }
-
+  
 private:
   //we forbid construction on the heap as good as we can
-  void* operator new( size_t size );
+  void * operator new( size_t size );
 };
 
-
 #endif /*kmfolder_h*/
