@@ -95,7 +95,7 @@ static const char configEntry[] = "State";
 
 KMMimePartTree::~KMMimePartTree()
 {
-  saveLayout( KMKernel::config(), configGroup, configEntry );
+  saveLayout( KMKernel::config().data(), configGroup, configEntry );
 }
 
 void KMMimePartTree::clearAndResetSortOrder()
@@ -108,7 +108,7 @@ void KMMimePartTree::clearAndResetSortOrder()
 
 void KMMimePartTree::restoreLayoutIfPresent()
 {
-  if ( restoreLayout( KMKernel::config(), configGroup, configEntry ) )
+  if ( restoreLayout( KMKernel::config().data(), configGroup, configEntry ) )
     return;
   // No configGroup or no configEntry.
   // Provide nice defaults on first show
@@ -121,7 +121,7 @@ void KMMimePartTree::showEvent( QShowEvent* e )
     // This seems to be the best way to provide reasonable defaults
     // for the column widths. We're triggered before the tree
     // is actually filled so fitting to contents is not an option (here).
-    // Having a fuction called from outside seems to be an overkill.
+    // Having a function called from outside seems to be an overkill.
     // QHeaderView::ResizeToContents can't be set since it disables
     // user resizing of columns...
     // So in the end, we provide heuristic defaults based on the width
@@ -298,7 +298,7 @@ void KMMimePartTree::startHandleAttachmentCommand( int action )
     return;
 
   partNode *node = static_cast<KMMimePartTreeItem *>( selected.first() )->node();
-  QString name = mReaderWin->tempFileUrlFromPartNode( node ).path();
+  QString name = mReaderWin->tempFileUrlFromPartNode( node ).toLocalFile();
 
   mReaderWin->prepareHandleAttachment( node->nodeId(), name );
   mReaderWin->slotHandleAttachment( action );
@@ -435,14 +435,14 @@ void KMMimePartTreeItem::correctSize()
 void KMMimePartTreeItem::setIconAndTextForType( const QString & mime )
 {
   QString mimetype = mime.toLower();
-  if ( mimetype.startsWith( "multipart/" ) ) {
+  if ( mimetype.startsWith( QLatin1String("multipart/") ) ) {
     setText( columnType, mimetype );
     setIcon( columnDescription, QIcon( SmallIcon("folder") ) );
   } else if ( mimetype == "application/octet-stream" ) {
     setText( columnType, i18n("Unspecified Binary Data") ); // do not show "Unknown"...
     setIcon( columnDescription, QIcon( SmallIcon("application-octet-stream") ) );
   } else {
-    KMimeType::Ptr mtp = KMimeType::mimeType( mimetype );
+    KMimeType::Ptr mtp = KMimeType::mimeType( mimetype, KMimeType::ResolveAliases );
     setText( columnType, ( mtp && !mtp->comment().isEmpty() ) ? mtp->comment() : mimetype );
     setIcon( columnDescription, QIcon( mtp ? KIconLoader::global()->loadMimeTypeIcon(mtp->iconName(), KIconLoader::Small) : SmallIcon("unknown") ) );
   }
