@@ -40,6 +40,8 @@
 #include <kdebug.h>
 #include <KFontAction>
 #include <KFontSizeAction>
+#include <kstatusbar.h>
+#include <khtml_part.h>
 #include "kmcommands.h"
 #include "kmenubar.h"
 #include "kmenu.h"
@@ -51,7 +53,7 @@
 #include "kmmsgdict.h"
 #include "mainfolderview.h"
 
-#include <kabc/stdaddressbook.h>
+#include <akonadi/contact/contactsearchjob.h>
 #include <kpimutils/email.h>
 
 KMReaderMainWin::KMReaderMainWin( bool htmlOverride, bool htmlLoadExtOverride,
@@ -96,7 +98,8 @@ KMReaderMainWin::KMReaderMainWin(KMMessagePart* aMsgPart,
 
 
 //-----------------------------------------------------------------------------
-void KMReaderMainWin::initKMReaderMainWin() {
+void KMReaderMainWin::initKMReaderMainWin()
+{
   setCentralWidget( mReaderWin );
   setupAccel();
   setupGUI( Keys | StatusBar | Create, "kmreadermainwin.rc" );
@@ -109,6 +112,9 @@ void KMReaderMainWin::initKMReaderMainWin() {
 
   connect( kmkernel, SIGNAL( configChanged() ),
            this, SLOT( slotConfigChanged() ) );
+
+  connect( mReaderWin->htmlPart(), SIGNAL( onURL( const QString& ) ),
+           statusBar(), SLOT( showMessage(const QString& ) ) );
 }
 
 //-----------------------------------------------------------------------------
@@ -180,6 +186,7 @@ void KMReaderMainWin::slotPrintMsg()
       mReaderWin->htmlOverride(), mReaderWin->htmlLoadExtOverride(),
       mReaderWin->isFixedFont(), mReaderWin->overrideEncoding() );
   command->setOverrideFont( mReaderWin->cssHelper()->bodyFont( mReaderWin->isFixedFont(), true /*printing*/ ) );
+  command->setAttachmentStrategy( mReaderWin->attachmentStrategy() );
   command->start();
 }
 
@@ -381,15 +388,19 @@ void KMReaderMainWin::slotMsgPopup( KMMessage &aMsg, const KUrl &aUrl, const QPo
         menu->addSeparator();
       }
       QString email =  KPIMUtils::firstEmailAddress( aUrl.path() );
-      KABC::AddressBook *addressBook = KABC::StdAddressBook::self( true );
-      KABC::Addressee::List addresseeList = addressBook->findByEmail( email );
+      Akonadi::ContactSearchJob *job = new Akonadi::ContactSearchJob( this );
+      job->setLimit( 1 );
+      job->setQuery( Akonadi::ContactSearchJob::Email, email );
+      job->exec();
+
+      KABC::Addressee::List addresseeList = job->contacts();
 
       if ( addresseeList.count() == 0 ) {
         menu->addAction( mReaderWin->addAddrBookAction() );
       } else {
         menu->addAction( mReaderWin->openAddrBookAction() );
       }
-      menu->addAction( mReaderWin->copyAction() );
+      menu->addAction( mReaderWin->copyURLAction() );
       copyAdded = true;
     } else {
       // popup on a not-mailto URL
@@ -437,6 +448,7 @@ void KMReaderMainWin::slotMsgPopup( KMMessage &aMsg, const KUrl &aUrl, const QPo
     menu->addSeparator();
     menu->addAction( mViewSourceAction );
     menu->addAction( mReaderWin->toggleFixFontAction() );
+    menu->addAction( mReaderWin->toggleMimePartTreeAction() );
     menu->addSeparator();
     menu->addAction( mPrintAction );
     menu->addAction( mSaveAsAction );
