@@ -59,15 +59,12 @@ using KMail::AccountManager;
  */
 KMSystemTray::KMSystemTray(QWidget *parent)
   : KSystemTrayIcon( parent),
-    mParentVisible( true ),
-    mPosOfMainWin( 0, 0 ),
-    mDesktopOfMainWin( 0 ),
     mMode( GlobalSettings::EnumSystemTrayPolicy::ShowOnUnread ),
     mCount( 0 ),
     mNewMessagesPopup( 0 ),
     mSendQueued( 0 )
 {
-  kDebug(5006) <<"Initting systray";
+  kDebug() << "Initting systray";
 
   mLastUpdate = time( 0 );
   mUpdateTimer = new QTimer( this );
@@ -76,17 +73,6 @@ KMSystemTray::KMSystemTray(QWidget *parent)
 
   mDefaultIcon = KIcon( "kmail" ).pixmap( 22 );
   setIcon( mDefaultIcon );
-#ifdef Q_WS_X11
-  KMMainWidget * mainWidget = kmkernel->getKMMainWidget();
-  if ( mainWidget ) {
-    QWidget * mainWin = mainWidget->topLevelWidget();
-    if ( mainWin ) {
-      mDesktopOfMainWin = KWindowSystem::windowInfo( mainWin->winId(),
-                                            NET::WMDesktop ).desktop();
-      mPosOfMainWin = mainWin->pos();
-    }
-  }
-#endif
   // register the applet with the kernel
   kmkernel->registerSystemTrayApplet( this );
 
@@ -152,7 +138,7 @@ void KMSystemTray::setMode(int newMode)
 {
   if(newMode == mMode) return;
 
-  kDebug(5006) <<"Setting systray mMode to" << newMode;
+  kDebug() << "Setting systray mMode to" << newMode;
   mMode = newMode;
 
   switch ( mMode ) {
@@ -167,7 +153,7 @@ void KMSystemTray::setMode(int newMode)
       show();
     break;
   default:
-    kDebug(5006) <<" Unknown systray mode" << mMode;
+    kDebug() << "Unknown systray mode" << mMode;
   }
 }
 
@@ -291,15 +277,12 @@ void KMSystemTray::foldersChanged()
  */
 void KMSystemTray::slotActivated( QSystemTrayIcon::ActivationReason reason )
 {
-  kDebug(5006) <<"trigger:" << reason;
+  kDebug() << "trigger:" << reason;
 
   // switch to kmail on left mouse button
   if( reason == QSystemTrayIcon::Trigger )
   {
-    if( mParentVisible && mainWindowIsOnCurrentDesktop() )
-      hideKMail();
-    else
-      showKMail();
+    kmkernel->toggleMainWin();
   }
 }
 
@@ -339,7 +322,7 @@ void KMSystemTray::slotContextMenuAboutToShow()
     mNewMessagesPopup->setTitle( i18n("New Messages In") );
     contextMenu()->insertAction( mSendQueued, mNewMessagesPopup->menuAction() );
 
-    kDebug(5006) <<"Folders added";
+    kDebug() << "Folders added";
   }
 }
 
@@ -358,91 +341,17 @@ QString KMSystemTray::prettyName(KMFolder * fldr)
     if((imap->account() != 0) &&
        (imap->account()->name() != 0) )
     {
-      kDebug(5006) <<"IMAP folder, prepend label with type";
+      kDebug() << "IMAP folder, prepend label with type";
       rvalue = imap->account()->name() + "->" + rvalue;
     }
   }
 
-  kDebug(5006) <<"Got label" << rvalue;
+  kDebug() << "Got label" << rvalue;
 
   return rvalue;
 }
 
 
-bool KMSystemTray::mainWindowIsOnCurrentDesktop()
-{
-#ifdef Q_WS_X11
-  KMMainWidget * mainWidget = kmkernel->getKMMainWidget();
-  if ( !mainWidget )
-    return false;
-
-  QWidget *mainWin = kmkernel->getKMMainWidget()->topLevelWidget();
-  if ( !mainWin )
-    return false;
-
-  return KWindowSystem::windowInfo( mainWin->winId(),
-                           NET::WMDesktop ).isOnCurrentDesktop();
-#else
-  return true;
-#endif
-}
-
-/**
- * Shows and raises the first KMMainWidget and
- * switches to the appropriate virtual desktop.
- */
-void KMSystemTray::showKMail()
-{
-  if (!kmkernel->getKMMainWidget())
-    return;
-  QWidget *mainWin = kmkernel->getKMMainWidget()->topLevelWidget();
-  assert(mainWin);
-#ifdef Q_WS_X11
-  if(mainWin)
-  {
-    KWindowInfo cur = KWindowSystem::windowInfo( mainWin->winId(), NET::WMDesktop );
-    if ( cur.valid() ) mDesktopOfMainWin = cur.desktop();
-    // switch to appropriate desktop
-    if ( mDesktopOfMainWin != NET::OnAllDesktops )
-      KWindowSystem::setCurrentDesktop( mDesktopOfMainWin );
-    if ( !mParentVisible ) {
-      if ( mDesktopOfMainWin == NET::OnAllDesktops )
-        KWindowSystem::setOnAllDesktops( mainWin->winId(), true );
-    }
-    KWindowSystem::activateWindow( mainWin->winId() );
-  }
-#endif
-  if ( !mParentVisible ) {
-    mainWin->move( mPosOfMainWin );
-    mainWin->show();
-    mParentVisible = true;
-  }
-  kmkernel->raise();
-
-  //Fake that the folders have changed so that the icon status is correct
-  foldersChanged();
-}
-
-void KMSystemTray::hideKMail()
-{
-  if (!kmkernel->getKMMainWidget())
-    return;
-  QWidget *mainWin = kmkernel->getKMMainWidget()->topLevelWidget();
-  assert(mainWin);
-  if(mainWin)
-  {
-    mPosOfMainWin = mainWin->pos();
-#ifdef Q_WS_X11
-    mDesktopOfMainWin = KWindowSystem::windowInfo( mainWin->winId(),
-                                          NET::WMDesktop ).desktop();
-    // iconifying is unnecessary, but it looks cooler
-    KWindowSystem::minimizeWindow( mainWin->winId() );
-#endif
-    mainWin->hide();
-    mParentVisible = false;
-  }
-}
-
 /**
  * Called on startup of the KMSystemTray and when the numUnreadMsgsChanged signal
  * is emitted for one of the watched folders.  Shows the system tray icon if there
@@ -456,7 +365,7 @@ void KMSystemTray::updateNewMessageNotification(KMFolder * fldr)
   if( !fldr ||
       fldr->folderType() == KMFolderTypeSearch )
   {
-    // kDebug(5006) <<"Null or a search folder, can't mess with that";
+    // kDebug() << "Null or a search folder, can't mess with that";
     return;
   }
 
@@ -500,7 +409,7 @@ void KMSystemTray::updateNewMessages()
     if ( unread > 0 ) {
       // Add folder to our internal store, or update unread count if already mapped
       mFoldersWithUnread.insert( fldr, unread );
-      //kDebug(5006) <<"There are now" << mFoldersWithUnread.count() <<" folders with unread";
+      //kDebug() << "There are now" << mFoldersWithUnread.count() << "folders with unread";
     }
 
     /*
@@ -522,7 +431,7 @@ void KMSystemTray::updateNewMessages()
     }
     else {
       if ( unread == 0 ) {
-        kDebug(5006) << "Removing folder from internal store" << fldr->name();
+        kDebug() << "Removing folder from internal store" << fldr->name();
 
         // Remove the folder from the internal store
         mFoldersWithUnread.remove(fldr);
@@ -559,7 +468,7 @@ void KMSystemTray::updateNewMessages()
  */
 void KMSystemTray::selectedAccount(int id)
 {
-  showKMail();
+  kmkernel->showMainWin();
 
   KMMainWidget * mainWidget = kmkernel->getKMMainWidget();
   if (!mainWidget)
