@@ -17,31 +17,43 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  */
-#include <config.h>
 
-#include <kuniqueapplication.h>
+#include <kontactinterface/pimuniqueapplication.h>
 #include <kglobal.h>
-#include <knotifyclient.h>
-#include <dcopclient.h>
 #include "kmkernel.h" //control center
+#include "kmmainwidget.h"
 #include "kmail_options.h"
 
 #include <kdebug.h>
 
+#include <akonadi/control.h>
+
 #undef Status // stupid X headers
 
 #include "aboutdata.h"
 
 #include "kmstartup.h"
 
+#ifdef Q_WS_WIN
+#include <unistd.h>
+#include <windows.h>
+#endif
+
 //-----------------------------------------------------------------------------
 
-class KMailApplication : public KUniqueApplication
+class KMailApplication : public KontactInterface::PimUniqueApplication
 {
 public:
-  KMailApplication() : KUniqueApplication() { };
+  KMailApplication() : KontactInterface::PimUniqueApplication(),
+                       mDelayedInstanceCreation( false ),
+                       mEventLoopReached( false ) { }
   virtual int newInstance();
   void commitData(QSessionManager& sm);
+  void setEventLoopReached();
+  void delayedInstanceCreation();
+protected:
+  bool mDelayedInstanceCreation;
+  bool mEventLoopReached;
 
 };
 
@@ -51,50 +63,72 @@ void KMailApplication::commitData(QSessionManager& sm) {
   KApplication::commitData( sm );
 }
 
+void KMailApplication::setEventLoopReached() {
+  mEventLoopReached = true;
+}
 
 int KMailApplication::newInstance()
 {
-  kdDebug(5006) << "KMailApplication::newInstance()" << endl;
+  kDebug();
+
+  // If the event loop hasn't been reached yet, the kernel is probably not
+  // fully initialized. Creating an instance would therefore fail, this is why
+  // that is delayed until delayedInstanceCreation() is called.
+  if ( !mEventLoopReached ) {
+    kDebug() << "Delaying instance creation.";
+    mDelayedInstanceCreation = true;
+    return 0;
+  }
+
   if (!kmkernel)
      return 0;
 
-  if (!kmkernel->firstInstance() || !kapp->isRestored())
+  if (!kmkernel->firstInstance() || !kapp->isSessionRestored())
     kmkernel->handleCommandLine( true );
-  kmkernel->setFirstInstance(FALSE);
+  kmkernel->setFirstInstance(false);
   return 0;
 }
 
+void KMailApplication::delayedInstanceCreation() {
+  if ( mDelayedInstanceCreation )
+    newInstance();
+}
+
 int main(int argc, char *argv[])
 {
   // WABA: KMail is a KUniqueApplication. Unfortunately this makes debugging
   // a bit harder: You should pass --nofork as commandline argument when using
   // a debugger. In gdb you can do this by typing "set args --nofork" before
   // typing "run".
-
+#if 0 // for testing KUniqueAppliaction on Windows
+  MessageBoxA(NULL, 
+             QString("main() %1 pid=%2").arg(argv[0]).arg(getpid()).toLatin1(), 
+             QString("main() \"%1\"").arg(argv[0]).toLatin1(), MB_OK|MB_ICONINFORMATION|MB_TASKMODAL);
+#endif
   KMail::AboutData about;
 
   KCmdLineArgs::init(argc, argv, &about);
-  KCmdLineArgs::addCmdLineOptions( kmail_options ); // Add kmail options
+  KCmdLineArgs::addCmdLineOptions( kmail_options() ); // Add kmail options
   if (!KMailApplication::start())
      return 0;
 
   KMailApplication app;
 
+  // Qt doesn't treat the system tray as a window, and therefore Qt would quit
+  // the event loop when an error message is clicked away while KMail is in the
+  // tray.
+  // Rely on KGlobal::ref() and KGlobal::deref() instead, like we did in KDE3.
+  // See http://bugs.kde.org/show_bug.cgi?id=163479
+  QApplication::setQuitOnLastWindowClosed( false );
+
   // import i18n data and icons from libraries:
   KMail::insertLibraryCataloguesAndIcons();
 
-  // Make sure that the KNotify Daemon is running (this is necessary for people
-  // using KMail without KDE)
-  KNotifyClient::startDaemon();
-
-  kapp->dcopClient()->suspend(); // Don't handle DCOP requests yet
-
   KMail::lockOrDie();
 
   //local, do the init
   KMKernel kmailKernel;
   kmailKernel.init();
-  kapp->dcopClient()->setDefaultObject( kmailKernel.objId() );
 
   // and session management
   kmailKernel.doSessionManagement();
@@ -104,10 +138,22 @@ int main(int argc, char *argv[])
 
   kmsetSignalHandler(kmsignalHandler);
 
-  kapp->dcopClient()->resume(); // Ok. We are ready for DCOP requests.
+  kmkernel->setupDBus(); // Ok. We are ready for D-Bus requests.
   kmkernel->setStartingUp( false ); // Starting up is finished
+
+  //If the instance hasn't been created yet, do that now
+  app.setEventLoopReached();
+  app.delayedInstanceCreation();
+
+  // Start Akonadi
+  if ( !Akonadi::Control::start( kmkernel->getKMMainWidget() ) ) {
+    //TODO: add message box after string freeze
+    kWarning() << "Unable to start Akonadi server, exit application";
+    return 1;
+  }
+
   // Go!
-  int ret = kapp->exec();
+  int ret = qApp->exec();
   // clean up
   kmailKernel.cleanup();
 
