@@ -25,19 +25,13 @@
 #include "kohelper.h"
 #include "koprefs.h"
 
-#include <kcal/incidence.h>
-#include <kcal/calendar.h>
-#include <kcal/calendarresources.h>
-#include <kcal/resourcecalendar.h>
+#include <KCal/CalendarResources>
 
-#include <kdatetime.h>
-#include <kdebug.h>
-
-#include <QColor>
+#include <QDate>
 
 QColor KOHelper::getTextColor( const QColor &c )
 {
-  float luminance = ( c.red() * 0.299 ) + ( c.green() * 0.587 ) + ( c.blue() * 0.114 );
+  double luminance = ( c.red() * 0.299 ) + ( c.green() * 0.587 ) + ( c.blue() * 0.114 );
   return ( luminance > 128.0 ) ? QColor( 0, 0, 0 ) : QColor( 255, 255, 255 );
 }
 
@@ -69,45 +63,12 @@ QColor KOHelper::resourceColor( KCal::Calendar *calendar,
         }
       }
     }
-//   } else {
-//     kDebug(5850) <<"resourceColor: Calendar is not a CalendarResources";
   }
   return resourceColor;
 }
 
-KDateTime KOHelper::toTimeSpec( const KDateTime &dt, KCal::Calendar *calendar )
+qint64 KOHelper::yearDiff( const QDate &start, const QDate &end )
 {
-  if ( dt.isDateOnly() ) {
-    return dt;
-  }
-
-  if ( calendar ) {
-    return dt.toTimeSpec( calendar->timeSpec() );
-  } else {
-    return dt.toTimeSpec( KOPrefs::instance()->timeSpec() );
-  }
-}
-
-QString KOHelper::resourceLabel( KCal::Calendar *calendar, KCal::Incidence *incidence )
-{
-  KCal::CalendarResources *calendarResource = dynamic_cast<KCal::CalendarResources*>( calendar );
-  if ( !calendarResource || ! incidence ) {
-    return QString();
-  }
-
-  KCal::ResourceCalendar *resourceCalendar = calendarResource->resource( incidence );
-  if ( resourceCalendar ) {
-    if ( !resourceCalendar->subresources().isEmpty() ) {
-      QString subRes = resourceCalendar->subresourceIdentifier( incidence );
-      if ( subRes.isEmpty() ) {
-        return resourceCalendar->resourceName();
-      } else {
-        return resourceCalendar->labelForSubresource( subRes );
-      }
-    }
-    return resourceCalendar->resourceName();
-  }
-
-  return QString();
+  return static_cast<qint64>( start.daysTo( end ) / 365.25 );
 }
 
