@@ -26,48 +26,36 @@
 */
 
 #include "kotodoview.h"
-#include "kotodoviewquicksearch.h"
-#include "kotodoviewquickaddline.h"
-#include "kotodomodel.h"
-#include "kotodoviewsortfilterproxymodel.h"
-#include "kotodoviewview.h"
-#include "kotododelegates.h"
-#include "koprefs.h"
-#include "koglobals.h"
-
-#include "kocorehelper.h"
-//#include "korganizer/baseview.h"
-//#include "actionmanager.h"
 #ifndef KORG_NOPRINTER
 #include "calprinter.h"
 #endif
+#include "kocorehelper.h"
+#include "koglobals.h"
+#include "koprefs.h"
+#include "kotododelegates.h"
+#include "kotodomodel.h"
+#include "kotodoviewquickaddline.h"
+#include "kotodoviewquicksearch.h"
+#include "kotodoviewview.h"
 
-#include <kcal/todo.h>
-#include <kcal/incidence.h>
+#include <libkdepim/kdatepickerpopup.h>
+using namespace KPIM;
 
-#include <kconfig.h>
-#include <kconfiggroup.h>
-#include <kdatepickerpopup.h>
-#include <kiconloader.h>
-#include <klineedit.h>
+#include <KCal/Incidence>
+#include <KCal/Todo>
 
 #include <QCheckBox>
-#include <QContextMenuEvent>
 #include <QGridLayout>
 #include <QHeaderView>
-#include <QList>
 #include <QMenu>
 #include <QTimer>
-#include <QModelIndex>
-
-using namespace KCal;
-using namespace KOrg;
-using namespace KPIM;
 
 KOTodoView::KOTodoView( Calendar *cal, QWidget *parent )
   : BaseView( cal, parent )
 {
   mModel = new KOTodoModel( calendar(), this );
+  connect( mModel, SIGNAL( expandIndex( const QModelIndex& ) ),
+           this, SLOT( expandIndex( const QModelIndex& ) ) );
   mProxyModel = new KOTodoViewSortFilterProxyModel( this );
   mProxyModel->setSourceModel( mModel );
   mProxyModel->setDynamicSortFilter( true );
@@ -123,16 +111,20 @@ KOTodoView::KOTodoView( Calendar *cal, QWidget *parent )
 
   mQuickAdd = new KOTodoViewQuickAddLine( this );
   mQuickAdd->setClearButtonShown( true );
-  QString clickText = i18n( "Click to add a new to-do" );
-  mQuickAdd->setToolTip( clickText );
   mQuickAdd->setVisible( KOPrefs::instance()->enableQuickTodo() );
   connect( mQuickAdd, SIGNAL(returnPressed(Qt::KeyboardModifiers)),
            this, SLOT(addQuickTodo(Qt::KeyboardModifiers)) );
 
   mFlatView = new QCheckBox( i18nc( "Checkbox to display todos not hirarchical",
                                     "Flat View" ), this );
-  mFlatView->setToolTip( i18n( "Display to-dos as list rather than as tree\n"
-                               "(i.e. without parental relationship displayed)" ) );
+  mFlatView->setToolTip(
+    i18nc( "@info:tooltip",
+           "Display to-dos in flat list instead of a tree" ) );
+  mFlatView->setWhatsThis(
+    i18nc( "@info:whatsthis",
+           "Checking this option will cause the to-dos to be displayed as a "
+           "flat list instead of a hierarchical tree; the parental "
+           "relationships are removed in the display." ) );
   connect( mFlatView, SIGNAL(toggled(bool)),
            this, SLOT(setFlatView(bool)) );
 
@@ -145,12 +137,6 @@ KOTodoView::KOTodoView( Calendar *cal, QWidget *parent )
 
   setLayout( layout );
 
-  // Do elided text after the layout is set so the width is properly computed
-  mQuickAdd->setClickMessage(
-    fontMetrics().elidedText(
-      clickText, Qt::ElideRight,
-      mQuickAdd->width() + mQuickAdd->clearButtonUsedSize().width() ) );
-
   // ---------------- POPUP-MENUS -----------------------
   mItemPopupMenu = new QMenu( this );
 
@@ -161,11 +147,17 @@ KOTodoView::KOTodoView( Calendar *cal, QWidget *parent )
     i18n( "&Edit..." ), this, SLOT(editTodo()) );
 
 #ifndef KORG_NOPRINTER
+  mItemPopupMenu->addSeparator();
   mItemPopupMenuItemOnlyEntries << mItemPopupMenu->addAction(
     KOGlobals::self()->smallIcon( "document-print" ),
     i18n( "&Print..." ), this, SLOT(printTodo()) );
+
+  mItemPopupMenuItemOnlyEntries << mItemPopupMenu->addAction(
+    KOGlobals::self()->smallIcon( "document-print-preview" ),
+    i18n( "Print Previe&w..." ), this, SLOT(printPreviewTodo()) );
 #endif
 
+  mItemPopupMenu->addSeparator();
   mItemPopupMenuItemOnlyEntries << mItemPopupMenu->addAction(
     KIconLoader::global()->loadIcon( "edit-delete", KIconLoader::NoGroup, KIconLoader::SizeSmall ),
     i18n( "&Delete" ), this, SLOT(deleteTodo()) );
@@ -253,6 +245,15 @@ KOTodoView::~KOTodoView()
 {
 }
 
+void KOTodoView::expandIndex( const QModelIndex &index )
+{
+  QModelIndex realIndex = mProxyModel->mapFromSource( index );
+  while ( realIndex.isValid() ) {
+    mView->expand( realIndex );
+    realIndex = mProxyModel->parent( realIndex );
+  }
+}
+
 void KOTodoView::setCalendar( Calendar *cal )
 {
   BaseView::setCalendar( cal );
@@ -268,15 +269,14 @@ Incidence::List KOTodoView::selectedIncidences()
   QModelIndexList selection = mView->selectionModel()->selectedRows();
 
   Q_FOREACH( const QModelIndex &mi, selection ) {
-    Todo *todo = static_cast<Todo *>( mi.data( KOTodoModel::TodoRole ).
-                                            value<void *>() );
+    Todo *todo = static_cast<Todo *>( mi.data( KOTodoModel::TodoRole ).value<void *>() );
     ret << todo;
   }
 
   return ret;
 }
 
-DateList KOTodoView::selectedDates()
+DateList KOTodoView::selectedIncidenceDates()
 {
   // The todo view only lists todo's. It's probably not a good idea to
   // return something about the selected todo here, because it has got
@@ -335,10 +335,12 @@ void KOTodoView::restoreLayout( KConfig *config, const QString &group, bool mini
     QTimer::singleShot( 0, this, SLOT(resizeColumnsToContent()) );
 
   } else {
-      for ( int i = 0; i < header->count()     &&
-                       i < columnOrder.size()  &&
-                       i < columnWidths.size() &&
-                       i < columnVisibility.size(); i++ ) {
+      for ( int i = 0;
+            i < header->count() &&
+            i < columnOrder.size() &&
+            i < columnWidths.size() &&
+            i < columnVisibility.size();
+            i++ ) {
       bool visible = columnVisibility[i].toBool();
       int width = columnWidths[i].toInt();
       int order = columnOrder[i].toInt();
@@ -373,11 +375,10 @@ void KOTodoView::showDates( const QDate &start, const QDate &end )
   Q_UNUSED( end );
 }
 
-void KOTodoView::showIncidences( const Incidence::List &incidenceList )
+void KOTodoView::showIncidences( const Incidence::List &incidenceList, const QDate &date )
 {
   Q_UNUSED( incidenceList );
-  // TODO: hmm, not sure how to do this...
-  kDebug() << "this is a stub";
+  Q_UNUSED( date );
 }
 
 void KOTodoView::updateView()
@@ -412,7 +413,7 @@ void KOTodoView::clearSelection()
 void KOTodoView::addQuickTodo( Qt::KeyboardModifiers modifiers )
 {
   if ( modifiers == Qt::NoModifier ) {
-    QModelIndex index = mModel->addTodo( mQuickAdd->text() );
+    QModelIndex index = mModel->addTodo( mQuickAdd->text(), QModelIndex(), mProxyModel->categories() );
 
     QModelIndexList selection = mView->selectionModel()->selectedRows();
     if ( selection.size() <= 1 ) {
@@ -427,7 +428,7 @@ void KOTodoView::addQuickTodo( Qt::KeyboardModifiers modifiers )
     if ( selection.size() != 1 ) {
       return;
     }
-    mModel->addTodo( mQuickAdd->text(), mProxyModel->mapToSource( selection[0] ) );
+    mModel->addTodo( mQuickAdd->text(), mProxyModel->mapToSource( selection[0] ), mProxyModel->categories() );
   } else {
     return;
   }
@@ -479,13 +480,17 @@ void KOTodoView::selectionChanged( const QItemSelection &selected,
   Q_UNUSED( deselected );
   QModelIndexList selection = selected.indexes();
   if ( selection.isEmpty() || !selection[0].isValid() ) {
-    emit incidenceSelected( 0 );
+    emit incidenceSelected( 0, QDate() );
     return;
   }
 
   Todo *todo = static_cast<Todo *>( selection[0].data( KOTodoModel::TodoRole ).value<void *>() );
 
-  emit incidenceSelected( todo );
+  if ( selectedIncidenceDates().isEmpty() ) {
+    emit incidenceSelected( todo, QDate() );
+  } else {
+    emit incidenceSelected( todo, selectedIncidenceDates().first() );
+  }
 }
 
 void KOTodoView::showTodo()
@@ -512,7 +517,7 @@ void KOTodoView::editTodo()
   emit editIncidenceSignal( todo );
 }
 
-void KOTodoView::printTodo()
+void KOTodoView::printTodo( bool preview )
 {
 #ifndef KORG_NOPRINTER
   QModelIndexList selection = mView->selectionModel()->selectedRows();
@@ -523,7 +528,7 @@ void KOTodoView::printTodo()
   Todo *todo = static_cast<Todo *>( selection[0].data( KOTodoModel::TodoRole ).value<void *>() );
 
   KOCoreHelper helper;
-  CalPrinter printer( this, BaseView::calendar(), &helper );
+  CalPrinter printer( this, BaseView::calendar(), &helper, true );
   connect( this, SIGNAL(configChanged()), &printer, SLOT(updateConfig()) );
 
   Incidence::List selectedIncidences;
@@ -537,11 +542,21 @@ void KOTodoView::printTodo()
   }
 
   printer.print( KOrg::CalPrinterBase::Incidence,
-                 todoDate.date(), todoDate.date(), selectedIncidences, false );
+                 todoDate.date(), todoDate.date(), selectedIncidences, preview );
 
 #endif
 }
 
+void KOTodoView::printTodo()
+{
+  printTodo( false );
+}
+
+void KOTodoView::printPreviewTodo()
+{
+  printTodo( true );
+}
+
 void KOTodoView::deleteTodo()
 {
   QModelIndexList selection = mView->selectionModel()->selectedRows();
@@ -648,7 +663,7 @@ void KOTodoView::setNewDate( const QDate &date )
     }
     todo->setDtDue( dt );
 
-    mChanger->changeIncidence( oldTodo, todo, KOGlobals::COMPLETION_MODIFIED );
+    mChanger->changeIncidence( oldTodo, todo, KOGlobals::COMPLETION_MODIFIED, this );
     mChanger->endChange( todo );
     delete oldTodo;
   } else {
@@ -676,9 +691,11 @@ void KOTodoView::setNewPercentage( QAction *action )
       todo->setPercentComplete( percentage );
     }
     if ( todo->recurs() && percentage == 100 ) {
-      mChanger->changeIncidence( oldTodo, todo, KOGlobals::COMPLETION_MODIFIED_WITH_RECURRENCE );
+      mChanger->changeIncidence( oldTodo, todo,
+                                 KOGlobals::COMPLETION_MODIFIED_WITH_RECURRENCE, this );
     } else {
-      mChanger->changeIncidence( oldTodo, todo, KOGlobals::COMPLETION_MODIFIED );
+      mChanger->changeIncidence( oldTodo, todo,
+                                 KOGlobals::COMPLETION_MODIFIED, this );
     }
     mChanger->endChange( todo );
     delete oldTodo;
@@ -700,7 +717,7 @@ void KOTodoView::setNewPriority( QAction *action )
     Todo *oldTodo = todo->clone();
     todo->setPriority( mPriority[action] );
 
-    mChanger->changeIncidence( oldTodo, todo, KOGlobals::PRIORITY_MODIFIED );
+    mChanger->changeIncidence( oldTodo, todo, KOGlobals::PRIORITY_MODIFIED, this );
     mChanger->endChange( todo );
     delete oldTodo;
   }
@@ -725,7 +742,7 @@ void KOTodoView::changedCategories( QAction *action )
     }
     categories.sort();
     todo->setCategories( categories );
-    mChanger->changeIncidence( oldTodo, todo, KOGlobals::CATEGORY_MODIFIED );
+    mChanger->changeIncidence( oldTodo, todo, KOGlobals::CATEGORY_MODIFIED, this );
     mChanger->endChange( todo );
     delete oldTodo;
   } else {
@@ -733,11 +750,21 @@ void KOTodoView::changedCategories( QAction *action )
   }
 }
 
-void KOTodoView::setFlatView( bool flatView ) {
+void KOTodoView::setFlatView( bool flatView )
+{
   mView->setRootIsDecorated( !flatView );
   mModel->setFlatView( flatView );
 }
 
+void KOTodoView::getHighlightMode( bool &highlightEvents,
+                                   bool &highlightTodos,
+                                   bool &highlightJournals )
+{
+  highlightTodos    = KOPrefs::instance()->mHighlightTodos;
+  highlightEvents   = !highlightTodos;
+  highlightJournals = false;
+}
+
 bool KOTodoView::usesFullWindow()
 {
   return KOPrefs::instance()->mFullViewTodo;
